/**
 * 
 */
package org.dataone.audit;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.lang.ArrayUtils;
import org.apache.log4j.Logger;
import org.dataone.configuration.Settings;


/**
 * @author rnahf
 *
 */
public class Main {

	static Logger logger = Logger.getLogger(Main.class.getName());
	
	/**
	 * The general facility for running various data-generating tools related
	 * to content-auditing of a DataONE environment.  The general approach is to 
	 * keep the tools fairly granular, so that they can be flexibly applied to 
	 * identify and isolate potential problems.  There will be two basic types of
	 * tools, those that are Identifier based, and those that are list based.
	 * 
	 * All queries except listObjects are authentication specific, so results will 
	 * vary depending on the client subject.
	 * 
	 * 
	 * @param args
	 */
	public static void main(String[] args) 
	{
		logger.info("Command Line ARGS:");
		for (String s: args) {
			logger.info(s);
		}
		logger.info("");
		
		/* name of the 'tool' to invoke */
		String action = null;
		
		/* flag for how to retrieve a list, for list based tools */
		boolean usePagingForLists = false;
		
		List<String> baseUrlList = new LinkedList<String>();
		
		// 
		Settings.getConfiguration().setProperty("certificate.truststore.useDefault",false);
		

		try {
			if (args.length == 0) {
				printUsage();
			} 
			else { 
				action = args[0];

			//TODO: posix parsing is not working, for some reason.  Need to fix at some point
			
//			// create the command line parser
//			CommandLineParser parser = new PosixParser();
//
//			// create the Options
//			Options options = new Options();
//			options.addOption( "h", "help", false, "print this message");
//			options.addOption( new Option("pagedListRetrieval", "if set, retrieve object or log record lists using a paged approach"));
//			
//			options.addOption( OptionBuilder.withLongOpt( "action" )
//											.withDescription("determines the action taken by the app")
//											.hasArg()
//											.withArgName("listIDs|listObjectInfos")
//											.create("action"));
//			
//			options.addOption( OptionBuilder.withLongOpt( "baseUrl" )
//					.withDescription("the baseUrl of the target node")
//					.hasArg()
//					.withArgName("<baseURL list>")
//					.create("u"));
//			
//			CommandLine line = parser.parse(options, args);
//			if (line.hasOption("h") || line.hasOption("help")) {
//				printStandardHelp("please choose the appropriate action and options", options);
//				return;
//			}
//			
//			// process the options 
//			
//			if (line.hasOption("action")) {
//				action = line.getOptionValue("action").toLowerCase();
//			} else {
//				printStandardHelp("please choose the appropriate action and options", options);
//				return;
//			}
//			
//			if (line.hasOption("baseUrl")) {
//				for (String url: line.getOptionValues("baseUrl")) {
//					baseUrlList.add(url);
//				}
//			}
//			
//
//			if (line.hasOption("pagedListRetrieval")) {
//				usePagingForLists = true;
//			}


			
			//////////////////////////////////////////////
			// do the work
				
				logger.info("start: " + new Date());

				List<String> report = null;
				if (action.equals("listMNs")) {
					if (args.length > 1) {
						report = AuditTools.listMemberNodeURLs(args[1], false);
						Collections.sort(report);
					}
				}
				else if (action.equals("listIDs")) {
					if (args.length > 1) {
						String[] formatTypes = (String[]) ArrayUtils.subarray(args, 2, args.length);
						report = AuditTools.listIdentifiers(args[1], formatTypes, usePagingForLists);
						Collections.sort(report);
					}
				}
				else if (action.equals("listObjectInfos")) {
					if (args.length > 1) {
						String[] formatTypes = (String[]) ArrayUtils.subarray(args, 2, args.length);
						report = AuditTools.listObjectInfos(args[1], formatTypes, usePagingForLists, false);
						Collections.sort(report);
					}
				}
				else if (action.equals("searchIDs")) {
					if (args.length > 1) {
						report = AuditTools.searchIDs(args[1]);
						Collections.sort(report);
					}
				}
				else if (action.equals("searchObjectInfos")) {
					if (args.length > 1) {
						report = AuditTools.searchObjectInfos(args[1], false);
						Collections.sort(report);
					}
				}
				else if (action.equals("listIDsAminusB")) {
					if (args.length > 2) {
						String[] formatTypes = (String[]) ArrayUtils.subarray(args, 3, args.length);
						report = AuditTools.idListAminusB(args[1], args[2], formatTypes);
						Collections.sort(report);
					}
				}
				else if (action.equals("objectInfosAminusB")) {
					if (args.length > 3) {
						boolean ignoreDates = args[3].equals("ignoreDates") ? true : false;
						report = AuditTools.objectInfosAminusB(args[1], args[2], null, ignoreDates);
						Collections.sort(report);
					} else {
						report = AuditTools.objectInfosAminusB(args[1], args[2], null, false);
						Collections.sort(report);
					}
				}
				else if (action.equals("verifyResolve")) {
					if (args.length > 2) {
						report = AuditTools.verifyResolve(args[1], args[2]);
					}
				}
				else if (action.equals("compareSysmeta")) {
					String[] burls = new String[args.length-2];
					System.arraycopy(args, 2, burls, 0, args.length - 2);
					report = AuditTools.compareSystemMetadata(args[1], burls, false);
				}
				else if (action.equals("compareSysmetaToSource")) {
					String[] burls = new String[args.length-2];
					System.arraycopy(args, 2, burls, 0, 1);
					report = AuditTools.compareSystemMetadata(args[1], burls, true);
				}
				else if (action.equals("verifyChecksum")) {
					String[] burls = new String[args.length-2];
					System.arraycopy(args, 2, burls, 0, 1);
					report = AuditTools.verifyChecksum(args[1], burls, false, false);
				}
				else if (action.equals("verifyChecksumVsSource")) {
					boolean recalculate = false;
					if (args.length > 2) {
						recalculate = args[3].equals("recalculate") ? true : false;
					}
					report = AuditTools.verifyChecksum(args[1], new String[]{args[2]}, true, recalculate);
				}
				else if (action.equals("findInconsistentIDs")) {
					report = AuditTools.compareCnObjectLists(args[1], false);
				}
				else if (action.equals("findInconsistentObjectInfos")) {
					report = AuditTools.compareCnObjectLists(args[1], true);
				}
				else {
					printUsage();
				}
			
				if (report != null) {
					int c = 0;
					for (String s: report) {
						System.out.println(s);
						c++;
					}
					logger.info("line count = " + c);
					logger.info("end: " + new Date());
				}
			}
		} 
		catch (Exception ex) {
			ex.printStackTrace();
			logger.error("unexpected exception" , ex);
		}
	}

	
	private static void printUsage() {
		System.out.println("java -jar d1_auditing_java <action> <args>");
		System.out.println("  where '<action> <args>' matches one of the following:");
		System.out.println("     listMNs  <cnBaseUrl> - list the baseUrls of all of the registered Member Nodes");
//		System.out.println("     listCNs  <cnBaseUrl> - list the baseUrls of all of the registered Member Nodes");
		System.out.println("     listIDs           <d1NodeBaseUrl> [formatType ]+ - list all of the identifiers returned from listObjects.");
		System.out.println("          the optional 'formatType' arguments will filter by the formatType (DATA, RESOURCE, METADATA)");
		System.out.println("          associated with the formatId of the returned objects");
		System.out.println("     listObjectInfos   <d1NodeBaseUrl> [formatType ]+  - returns a tabular view of objectInfos from listObjects.");
		System.out.println("          the optional 'formatType' arguments will filter by the formatType (DATA, RESOURCE, METADATA)");
		System.out.println("          associated with the formatId of the returned objects");
		System.out.println("     searchIDs         <d1NodeBaseUrl>  - lists all of the identifiers returned from cn.search");
		System.out.println("     searchObjectInfos <d1NodeBaseUrl>  - returns a tabular view of objectInfos from cn.search");
		System.out.println("     listIDsAminusB        <d1NodeBaseUrl-A> <d1NodeBaseUrl-B>  [formatType ]+ - list the identifiers in Node A but not Node B");
		System.out.println("          the optional 'formatType' argument will filter by the formatType (DATA, RESOURCE, METADATA)");
		System.out.println("          associated with the formatId of the returned objects in the 'A' list");
		System.out.println("     objectInfosAminusB    <d1NodeBaseUrl-A> <d1NodeBaseUrl-B> [ignoreDates] "); 
		System.out.println("          list the objectInfos in Node A but not Node B.  with the ignoreDates flag,");
		System.out.println("          objectInfos differing only by the sysmeta modified date are considered the same");
		System.out.println("");
		System.out.println("     findInconsistentIDs  <cnBaseUrl>   - list all of the identifiers not found on all CNs");
		System.out.println("     findInconsistentObjectInfos  <cnBaseUrl> - list all of the objectInfos not found on all CNs");
		System.out.println("");
		System.out.println("     verifyResolve      <identifier> <cnBaseUrl> - verifies that all replicas are retrievable");
		System.out.println("     compareSysmeta   <identifier> <baseUrl> <baseUrl> ... - reports the fields of the sysmeta that differ");
		System.out.println("     compareSysmetaToSource    <identifier> <cnBaseUrl>   - looks up the authoritativeMN");
		System.out.println("          from the systemMetadata found on the CN as node to compare against");
		System.out.println("     verifyChecksum   <identifier> <baseUrl> <baseUrl> ... - reports the checksum fields that differ");
		System.out.println("     verifyChecksumVsSource    <identifier> <cnBaseUrl>   - looks up the authoritativeMN ");
		System.out.println("          from the systemMetadata found on the CN as node to compare against");
		System.out.println("");
		System.out.println("  and <baseurl> can be replaced with a valid filepath or '-' (for stdin)");
		System.out.println("");
		System.out.println("* the 'finding' and 'listing' tests will need additional heap space to complete  (-Xmx512M, for example)");
		
	}
	
	
	private static InputStream getInputStream(String filePath) throws FileNotFoundException {

		if (filePath.equals('-')) 
			return System.in;
		
		FileInputStream fis = new FileInputStream(new File(filePath));
		return fis;
	}
	

	
	
	
	private static void printStandardHelp(String footer, Options options) {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("java -jar d1_auditing_java", 
							"This program facilitates the running of audting tools and tests.\n" +
							"The 'action' option determines which tool to execute.\n",
							options,
							"\n" + footer);
	}

}
		

